@@ -1,27 +1,73 @@
+import cython
+from Cython import __version__
+
 from glob import glob
 import re, os, sys
-from cython import set
+try:
+    import gzip
+    gzip_open = gzip.open
+    gzip_ext = '.gz'
+except ImportError:
+    gzip_open = open
+    gzip_ext = ''
+import shutil
+import subprocess
+
+try:
+    import hashlib
+except ImportError:
+    import md5 as hashlib
 
 
 from distutils.extension import Extension
 
 from Cython import Utils
+from Cython.Utils import cached_function, cached_method, path_exists
 from Cython.Compiler.Main import Context, CompilationOptions, default_options
+    
+join_path = cached_function(os.path.join)
 
-# Unfortunately, Python 2.3 doesn't support decorators.
-def cached_method(f):
-    cache_name = '__%s_cache' % f.__name__
-    def wrapper(self, *args):
-        cache = getattr(self, cache_name, None)
-        if cache is None:
-            cache = {}
-            setattr(self, cache_name, cache)
-        if args in cache:
-            return cache[args]
-        res = cache[args] = f(self, *args)
-        return res
-    return wrapper
+if sys.version_info[0] < 3:
+    # stupid Py2 distutils enforces str type in list of sources
+    _fs_encoding = sys.getfilesystemencoding()
+    if _fs_encoding is None:
+        _fs_encoding = sys.getdefaultencoding()
+    def encode_filename_in_py2(filename):
+        if isinstance(filename, unicode):
+            return filename.encode(_fs_encoding)
+        return filename
+else:
+    def encode_filename_in_py2(filename):
+        return filename
 
+def extended_iglob(pattern):
+    if '**/' in pattern:
+        seen = set()
+        first, rest = pattern.split('**/', 1)
+        if first:
+            first = glob(first+'/')
+        else:
+            first = ['']
+        for root in first:
+            for path in extended_iglob(join_path(root, rest)):
+                if path not in seen:
+                    seen.add(path)
+                    yield path
+            for path in extended_iglob(join_path(root, '*', '**', rest)):
+                if path not in seen:
+                    seen.add(path)
+                    yield path
+    else:
+        for path in glob(pattern):
+            yield path
+
+@cached_function
+def file_hash(filename):
+    path = os.path.normpath(filename.encode("UTF-8"))
+    m = hashlib.md5(str(len(path)) + ":")
+    m.update(path)
+    m.update(open(filename).read())
+    return m.hexdigest()
 
 def parse_list(s):
     """
@@ -68,6 +114,7 @@ distutils_settings = {
     'language':             transitive_str,
 }
 
+@cython.locals(start=long, end=long)
 def line_iter(source):
     start = 0
     while True:
@@ -146,7 +193,8 @@ class DistutilsInfo(object):
             resolved.values[key] = value
         return resolved
 
-
+@cython.locals(start=long, q=long, single_q=long, double_q=long, hash_mark=long,
+               end=long, k=long, counter=long, quote_len=long)
 def strip_string_literals(code, prefix='__Pyx_L'):
     """
     Normalizes every string literal to be of the form '__Pyx_Lxxx',
@@ -158,11 +206,16 @@ def strip_string_literals(code, prefix='__Pyx_L'):
     counter = 0
     start = q = 0
     in_quote = False
-    raw = False
+    hash_mark = single_q = double_q = -1
+    code_len = len(code)
+    
     while True:
-        hash_mark = code.find('#', q)
-        single_q = code.find("'", q)
-        double_q = code.find('"', q)
+        if hash_mark < q:
+            hash_mark = code.find('#', q)
+        if single_q < q:
+            single_q = code.find("'", q)
+        if double_q < q:
+            double_q = code.find('"', q)
         q = min(single_q, double_q)
         if q == -1: q = max(single_q, double_q)
 
@@ -173,19 +226,22 @@ def strip_string_literals(code, prefix='__Pyx_L'):
 
         # Try to close the quote.
         elif in_quote:
-            if code[q-1] == '\\' and not raw:
+            if code[q-1] == u'\\':
                 k = 2
-                while q >= k and code[q-k] == '\\':
+                while q >= k and code[q-k] == u'\\':
                     k += 1
                 if k % 2 == 0:
                     q += 1
                     continue
-            if code[q:q+len(in_quote)] == in_quote:
+            if code[q] == quote_type and (quote_len == 1 or (code_len > q + 2 and quote_type == code[q+1] == code[q+2])):
                 counter += 1
                 label = "%s%s_" % (prefix, counter)
-                literals[label] = code[start+len(in_quote):q]
-                new_code.append("%s%s%s" % (in_quote, label, in_quote))
-                q += len(in_quote)
+                literals[label] = code[start+quote_len:q]
+                full_quote = code[q:q+quote_len]
+                new_code.append(full_quote)
+                new_code.append(label)
+                new_code.append(full_quote)
+                q += quote_len
                 in_quote = False
                 start = q
             else:
@@ -193,116 +249,139 @@ def strip_string_literals(code, prefix='__Pyx_L'):
 
         # Process comment.
         elif -1 != hash_mark and (hash_mark < q or q == -1):
-            end = code.find('\n', hash_mark)
-            if end == -1:
-                end = None
             new_code.append(code[start:hash_mark+1])
+            end = code.find('\n', hash_mark)
             counter += 1
             label = "%s%s_" % (prefix, counter)
-            literals[label] = code[hash_mark+1:end]
+            if end == -1:
+                end_or_none = None
+            else:
+                end_or_none = end
+            literals[label] = code[hash_mark+1:end_or_none]
             new_code.append(label)
-            if end is None:
+            if end == -1:
                 break
-            q = end
-            start = q
+            start = q = end
 
         # Open the quote.
         else:
-            raw = False
-            if len(code) >= q+3 and (code[q+1] == code[q] == code[q+2]):
-                in_quote = code[q]*3
+            if code_len >= q+3 and (code[q] == code[q+1] == code[q+2]):
+                quote_len = 3
             else:
-                in_quote = code[q]
-            end = marker = q
-            while marker > 0 and code[marker-1] in 'rRbBuU':
-                if code[marker-1] in 'rR':
-                    raw = True
-                marker -= 1
-            new_code.append(code[start:end])
+                quote_len = 1
+            in_quote = True
+            quote_type = code[q]
+            new_code.append(code[start:q])
             start = q
-            q += len(in_quote)
+            q += quote_len
 
     return "".join(new_code), literals
 
 
+dependancy_regex = re.compile(r"(?:^from +([0-9a-zA-Z_.]+) +cimport)|"
+                              r"(?:^cimport +([0-9a-zA-Z_.]+)\b)|"
+                              r"(?:^cdef +extern +from +['\"]([^'\"]+)['\"])|"
+                              r"(?:^include +['\"]([^'\"]+)['\"])", re.M)
+
+def normalize_existing(base_path, rel_paths):
+    return normalize_existing0(os.path.dirname(base_path), tuple(set(rel_paths)))
+
+@cached_function
+def normalize_existing0(base_dir, rel_paths):
+    filtered = []
+    for rel in rel_paths:
+        path = join_path(base_dir, rel)
+        if os.path.exists(path):
+            filtered.append(os.path.normpath(path))
+    return filtered
+
+@cached_function
 def parse_dependencies(source_filename):
     # Actual parsing is way to slow, so we use regular expressions.
     # The only catch is that we must strip comments and string
     # literals ahead of time.
-    fh = Utils.open_source_file(source_filename, "rU")
+    fh = Utils.open_source_file(source_filename, "rU", error_handling='ignore')
     try:
         source = fh.read()
     finally:
         fh.close()
     distutils_info = DistutilsInfo(source)
     source, literals = strip_string_literals(source)
-    source = source.replace('\\\n', ' ')
-    if '\t' in source:
-        source = source.replace('\t', ' ')
+    source = source.replace('\\\n', ' ').replace('\t', ' ')
+
     # TODO: pure mode
-    dependancy = re.compile(r"(cimport +([0-9a-zA-Z_.]+)\b)|(from +([0-9a-zA-Z_.]+) +cimport)|(include +'([^']+)')|(cdef +extern +from +'([^']+)')")
     cimports = []
     includes = []
     externs  = []
-    for m in dependancy.finditer(source):
-        groups = m.groups()
-        if groups[0]:
-            cimports.append(groups[1])
-        elif groups[2]:
-            cimports.append(groups[3])
-        elif groups[4]:
-            includes.append(literals[groups[5]])
+    for m in dependancy_regex.finditer(source):
+        cimport_from, cimport, extern, include = m.groups()
+        if cimport_from:
+            cimports.append(cimport_from)
+        elif cimport:
+            cimports.append(cimport)
+        elif extern:
+            externs.append(literals[extern])
         else:
-            externs.append(literals[groups[7]])
+            includes.append(literals[include])
     return cimports, includes, externs, distutils_info
 
 
 class DependencyTree(object):
 
-    def __init__(self, context):
+    def __init__(self, context, quiet=False):
         self.context = context
+        self.quiet = quiet
         self._transitive_cache = {}
 
-    #@cached_method
     def parse_dependencies(self, source_filename):
         return parse_dependencies(source_filename)
-    parse_dependencies = cached_method(parse_dependencies)
 
-    #@cached_method
+    @cached_method
+    def included_files(self, filename):
+        # This is messy because included files are textually included, resolving
+        # cimports (but not includes) relative to the including file.
+        all = set()
+        for include in self.parse_dependencies(filename)[1]:
+            include_path = join_path(os.path.dirname(filename), include)
+            if not path_exists(include_path):
+                include_path = self.context.find_include_file(include, None)
+            if include_path:
+                if '.' + os.path.sep in include_path:
+                    include_path = os.path.normpath(include_path)
+                all.add(include_path)
+                all.update(self.included_files(include_path))
+            elif not self.quiet:
+                print("Unable to locate '%s' referenced from '%s'" % (filename, include))
+        return all
+    
+    @cached_method
     def cimports_and_externs(self, filename):
+        # This is really ugly. Nested cimports are resolved with respect to the
+        # includer, but includes are resolved with respect to the includee.
         cimports, includes, externs = self.parse_dependencies(filename)[:3]
         cimports = set(cimports)
         externs = set(externs)
-        for include in includes:
-            include_path = os.path.join(os.path.dirname(filename), include)
-            if not os.path.exists(include_path):
-                include_path = self.context.find_include_file(include, None)
-            if include_path:
-                a, b = self.cimports_and_externs(include_path)
-                cimports.update(a)
-                externs.update(b)
-            else:
-                print("Unable to locate '%s' referenced from '%s'" % (filename, include))
-        return tuple(cimports), tuple(externs)
-    cimports_and_externs = cached_method(cimports_and_externs)
+        for include in self.included_files(filename):
+            included_cimports, included_externs = self.cimports_and_externs(include)
+            cimports.update(included_cimports)
+            externs.update(included_externs)
+        return tuple(cimports), normalize_existing(filename, externs)
 
     def cimports(self, filename):
         return self.cimports_and_externs(filename)[0]
 
-    #@cached_method
+    @cached_method
     def package(self, filename):
-        dir = os.path.dirname(os.path.abspath(filename))
-        if dir != filename and os.path.exists(os.path.join(dir, '__init__.py')):
+        dir = os.path.dirname(os.path.abspath(str(filename)))
+        if dir != filename and path_exists(join_path(dir, '__init__.py')):
             return self.package(dir) + (os.path.basename(dir),)
         else:
             return ()
-    package = cached_method(package)
 
-    #@cached_method
+    @cached_method
     def fully_qualifeid_name(self, filename):
         module = os.path.splitext(os.path.basename(filename))[0]
         return '.'.join(self.package(filename) + (module,))
-    fully_qualifeid_name = cached_method(fully_qualifeid_name)
 
     def find_pxd(self, module, filename=None):
         if module[0] == '.':
@@ -315,41 +394,64 @@ class DependencyTree(object):
         return self.context.find_pxd_file(module, None)
     find_pxd = cached_method(find_pxd)
 
-    #@cached_method
+    @cached_method
     def cimported_files(self, filename):
-        if filename[-4:] == '.pyx' and os.path.exists(filename[:-4] + '.pxd'):
-            self_pxd = [filename[:-4] + '.pxd']
+        if filename[-4:] == '.pyx' and path_exists(filename[:-4] + '.pxd'):
+            pxd_list = [filename[:-4] + '.pxd']
         else:
-            self_pxd = []
-        a = self.cimports(filename)
-        b = filter(None, [self.find_pxd(m, filename) for m in self.cimports(filename)])
-        if len(a) - int('cython' in a) != len(b):
-            print("missing cimport", filename)
-            print("\n\t".join(a))
-            print("\n\t".join(b))
-        return tuple(self_pxd + filter(None, [self.find_pxd(m, filename) for m in self.cimports(filename)]))
-    cimported_files = cached_method(cimported_files)
+            pxd_list = []
+        for module in self.cimports(filename):
+            if module[:7] == 'cython.' or module == 'cython':
+                continue
+            pxd_file = self.find_pxd(module, filename)
+            if pxd_file is not None:
+                pxd_list.append(pxd_file)
+            elif not self.quiet:
+                print("missing cimport in module '%s': %s" % (module, filename))
+        return tuple(pxd_list)
 
+    @cached_method
     def immediate_dependencies(self, filename):
-        all = list(self.cimported_files(filename))
-        for extern in sum(self.cimports_and_externs(filename), ()):
-            all.append(os.path.normpath(os.path.join(os.path.dirname(filename), extern)))
-        return tuple(all)
+        all = set([filename])
+        all.update(self.cimported_files(filename))
+        all.update(self.included_files(filename))
+        return all
 
-    #@cached_method
+    def all_dependencies(self, filename):
+        return self.transitive_merge(filename, self.immediate_dependencies, set.union)
+
+    @cached_method
     def timestamp(self, filename):
         return os.path.getmtime(filename)
-    timestamp = cached_method(timestamp)
 
     def extract_timestamp(self, filename):
-        # TODO: .h files from extern blocks
         return self.timestamp(filename), filename
 
     def newest_dependency(self, filename):
-        return self.transitive_merge(filename, self.extract_timestamp, max)
+        return max([self.extract_timestamp(f) for f in self.all_dependencies(filename)])
+
+    def transitive_fingerprint(self, filename, extra=None):
+        try:
+            m = hashlib.md5(__version__)
+            m.update(file_hash(filename))
+            for x in sorted(self.all_dependencies(filename)):
+                if os.path.splitext(x)[1] not in ('.c', '.cpp', '.h'):
+                    m.update(file_hash(x))
+            if extra is not None:
+                m.update(str(extra))
+            return m.hexdigest()
+        except IOError:
+            return None
 
     def distutils_info0(self, filename):
-        return self.parse_dependencies(filename)[3]
+        info = self.parse_dependencies(filename)[3]
+        externs = self.cimports_and_externs(filename)[1]
+        if externs:
+            if 'depends' in info.values:
+                info.values['depends'] = list(set(info.values['depends']).union(set(externs)))
+            else:
+                info.values['depends'] = list(externs)
+        return info
 
     def distutils_info(self, filename, aliases=None, base=None):
         return (self.transitive_merge(filename, self.distutils_info0, DistutilsInfo.merge)
@@ -390,23 +492,24 @@ class DependencyTree(object):
             del stack[node]
 
 _dep_tree = None
-def create_dependency_tree(ctx=None):
+def create_dependency_tree(ctx=None, quiet=False):
     global _dep_tree
     if _dep_tree is None:
         if ctx is None:
             ctx = Context(["."], CompilationOptions(default_options))
-        _dep_tree = DependencyTree(ctx)
+        _dep_tree = DependencyTree(ctx, quiet=quiet)
     return _dep_tree
 
 # This may be useful for advanced users?
-def create_extension_list(patterns, exclude=[], ctx=None, aliases=None):
+def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=False):
+    explicit_modules = set([m.name for m in patterns if isinstance(m, Extension)])
     seen = set()
-    deps = create_dependency_tree(ctx)
+    deps = create_dependency_tree(ctx, quiet=quiet)
     to_exclude = set()
     if not isinstance(exclude, list):
         exclude = [exclude]
     for pattern in exclude:
-        to_exclude.update(glob(pattern))
+        to_exclude.update(extended_iglob(pattern))
     if not isinstance(patterns, list):
         patterns = [patterns]
     module_list = []
@@ -429,12 +532,14 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None):
             exn_type = template.__class__
         else:
             raise TypeError(pattern)
-        for file in glob(filepattern):
+        for file in extended_iglob(filepattern):
             if file in to_exclude:
                 continue
             pkg = deps.package(file)
             if '*' in name:
                 module_name = deps.fully_qualifeid_name(file)
+                if module_name in explicit_modules:
+                    continue
             else:
                 module_name = name
             if module_name not in seen:
@@ -446,6 +551,13 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None):
                 sources = [file]
                 if template is not None:
                     sources += template.sources[1:]
+                if 'sources' in kwds:
+                    # allow users to add .c files etc.
+                    for source in kwds['sources']:
+                        source = encode_filename_in_py2(source)
+                        if source not in sources:
+                            sources.append(source)
+                    del kwds['sources']
                 module_list.append(exn_type(
                         name=module_name,
                         sources=sources,
@@ -455,7 +567,29 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None):
     return module_list
 
 # This is the user-exposed entry point.
-def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, force=False, **options):
+def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, force=False,
+              exclude_failures=False, **options):
+    """
+    Compile a set of source modules into C/C++ files and return a list of distutils
+    Extension objects for them.
+
+    As module list, pass either a glob pattern, a list of glob patterns or a list of
+    Extension objects.  The latter allows you to configure the extensions separately
+    through the normal distutils options.
+
+    When using glob patterns, you can exclude certain module names explicitly
+    by passing them into the 'exclude' option.
+
+    For parallel compilation, set the 'nthreads' option to the number of
+    concurrent builds.
+
+    For a broad 'try to compile' mode that ignores compilation failures and
+    simply excludes the failed extensions, pass 'exclude_failures=True'. Note
+    that this only really makes sense for compiling .py files which can also
+    be used without compilation.
+
+    Additional compilation options can be passed as keyword arguments.
+    """
     if 'include_path' not in options:
         options['include_path'] = ['.']
     c_options = CompilationOptions(**options)
@@ -465,8 +599,10 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
         module_list,
         exclude=exclude,
         ctx=ctx,
+        quiet=quiet,
         aliases=aliases)
-    deps = create_dependency_tree(ctx)
+    deps = create_dependency_tree(ctx, quiet=quiet)
+    modules_by_cfile = {}
     to_compile = []
     for m in module_list:
         new_sources = []
@@ -483,6 +619,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                     c_timestamp = os.path.getmtime(c_file)
                 else:
                     c_timestamp = -1
+                    
                 # Priority goes first to modified files, second to direct
                 # dependents, and finally to indirect dependents.
                 if c_timestamp < deps.timestamp(source):
@@ -497,11 +634,24 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                             print("Compiling %s because it changed." % source)
                         else:
                             print("Compiling %s because it depends on %s." % (source, dep))
-                    to_compile.append((priority, source, c_file, options))
+                    if not force and hasattr(options, 'cache'):
+                        extra = m.language
+                        fingerprint = deps.transitive_fingerprint(source, extra)
+                    else:
+                        fingerprint = None
+                    to_compile.append((priority, source, c_file, fingerprint, quiet,
+                                       options, not exclude_failures))
                 new_sources.append(c_file)
+                if c_file not in modules_by_cfile:
+                    modules_by_cfile[c_file] = [m]
+                else:
+                    modules_by_cfile[c_file].append(m)
             else:
                 new_sources.append(source)
         m.sources = new_sources
+    if hasattr(options, 'cache'):
+        if not os.path.exists(options.cache):
+            os.mkdir(options.cache)
     to_compile.sort()
     if nthreads:
         # Requires multiprocessing (or Python >= 2.6)
@@ -513,17 +663,51 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
             print("multiprocessing required for parallel cythonization")
             nthreads = 0
     if not nthreads:
-        for priority, pyx_file, c_file, options in to_compile:
-            cythonize_one(pyx_file, c_file, quiet, options)
+        for args in to_compile:
+            cythonize_one(*args[1:])
+    if exclude_failures:
+        failed_modules = set()
+        for c_file, modules in modules_by_cfile.iteritems():
+            if not os.path.exists(c_file):
+                failed_modules.update(modules)
+        for module in failed_modules:
+            module_list.remove(module)
+    if hasattr(options, 'cache'):
+        cleanup_cache(options.cache, getattr(options, 'cache_size', 1024 * 1024 * 100))
     return module_list
 
 # TODO: Share context? Issue: pyx processing leaks into pxd module
-def cythonize_one(pyx_file, c_file, quiet, options=None):
+def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True):
     from Cython.Compiler.Main import compile, default_options
     from Cython.Compiler.Errors import CompileError, PyrexError
 
+    if fingerprint:
+        if not os.path.exists(options.cache):
+            try:
+                os.mkdir(options.cache)
+            except:
+                if not os.path.exists(options.cache):
+                    raise
+        # Cython-generated c files are highly compressible.
+        # (E.g. a compression ratio of about 10 for Sage).
+        fingerprint_file = join_path(
+            options.cache, "%s-%s%s" % (os.path.basename(c_file), fingerprint, gzip_ext))
+        if os.path.exists(fingerprint_file):
+            if not quiet:
+                print("Found compiled %s in cache" % pyx_file)
+            os.utime(fingerprint_file, None)
+            g = gzip_open(fingerprint_file, 'rb')
+            try:
+                f = open(c_file, 'wb')
+                try:
+                    shutil.copyfileobj(g, f)
+                finally:
+                    f.close()
+            finally:
+                g.close()
+            return
     if not quiet:
-        print "Cythonizing %s" % pyx_file
+        print("Cythonizing %s" % pyx_file)
     if options is None:
         options = CompilationOptions(default_options)
     options.output_file = c_file
@@ -534,10 +718,53 @@ def cythonize_one(pyx_file, c_file, quiet, options=None):
         if result.num_errors > 0:
             any_failures = 1
     except (EnvironmentError, PyrexError), e:
-        sys.stderr.write(str(e) + '\n')
+        sys.stderr.write('%s\n' % e)
+        any_failures = 1
+    except Exception:
+        if raise_on_failure:
+            raise
+        import traceback
+        traceback.print_exc()
         any_failures = 1
     if any_failures:
-        raise CompileError(None, pyx_file)
+        if raise_on_failure:
+            raise CompileError(None, pyx_file)
+        elif os.path.exists(c_file):
+            os.remove(c_file)
+    elif fingerprint:
+        f = open(c_file, 'rb')
+        try:
+            g = gzip_open(fingerprint_file, 'wb')
+            try:
+                shutil.copyfileobj(f, g)
+            finally:
+                g.close()
+        finally:
+            f.close()
 
 def cythonize_one_helper(m):
     return cythonize_one(*m[1:])
+
+def cleanup_cache(cache, target_size, ratio=.85):
+    try:
+        p = subprocess.Popen(['du', '-s', '-k', os.path.abspath(cache)], stdout=subprocess.PIPE)
+        res = p.wait()
+        if res == 0:
+            total_size = 1024 * int(p.stdout.read().strip().split()[0])
+            if total_size < target_size:
+                return
+    except (OSError, ValueError):
+        pass
+    total_size = 0
+    all = []
+    for file in os.listdir(cache):
+        path = join_path(cache, file)
+        s = os.stat(path)
+        total_size += s.st_size
+        all.append((s.st_atime, s.st_size, path))
+    if total_size > target_size:
+        for time, size, file in reversed(sorted(all)):
+            os.unlink(file)
+            total_size -= size
+            if total_size < target_size * ratio:
+                break
