@@ -31,39 +31,7 @@ proto = '''
 #define __Pyx_abs_long(x) __Pyx_abs_int(x)
 ''')
 
-iter_next_utility_code = UtilityCode(
-proto = """
-#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL);
-static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *); /*proto*/
-""",
-# copied from Py3's builtin_next()
-impl = '''
-static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
-    PyObject* next;
-    if (unlikely(!PyIter_Check(iterator))) {
-        PyErr_Format(PyExc_TypeError,
-            "%.200s object is not an iterator", iterator->ob_type->tp_name);
-        return NULL;
-    }
-    next = (*(Py_TYPE(iterator)->tp_iternext))(iterator);
-    if (likely(next)) {
-        return next;
-    } else if (defval) {
-        if (PyErr_Occurred()) {
-            if(!PyErr_ExceptionMatches(PyExc_StopIteration))
-                return NULL;
-            PyErr_Clear();
-        }
-        Py_INCREF(defval);
-        return defval;
-    } else if (PyErr_Occurred()) {
-        return NULL;
-    } else {
-        PyErr_SetNone(PyExc_StopIteration);
-        return NULL;
-    }
-}
-''')
+iter_next_utility_code = UtilityCode.load_cached("IterNext", "ObjectHandling.c")
 
 getattr3_utility_code = UtilityCode(
 proto = """
@@ -85,134 +53,9 @@ bad:
 }
 """)
 
-globals_utility_code = UtilityCode(
-# This is a stub implementation until we have something more complete.
-# Currently, we only handle the most common case of a read-only dict
-# of Python names.  Supporting cdef names in the module and write
-# access requires a rewrite as a dedicated class.
-proto = """
-static PyObject* __Pyx_Globals(void); /*proto*/
-""",
-impl = '''
-static PyObject* __Pyx_Globals() {
-    Py_ssize_t i;
-    /*PyObject *d;*/
-    PyObject *names = NULL;
-    PyObject *globals = PyObject_GetAttrString(%(MODULE)s, "__dict__");
-    if (!globals) {
-        PyErr_SetString(PyExc_TypeError,
-            "current module must have __dict__ attribute");
-        goto bad;
-    }
-    names = PyObject_Dir(%(MODULE)s);
-    if (!names)
-        goto bad;
-    for (i = 0; i < PyList_GET_SIZE(names); i++) {
-        PyObject* name = PyList_GET_ITEM(names, i);
-        if (!PyDict_Contains(globals, name)) {
-            PyObject* value = PyObject_GetAttr(%(MODULE)s, PyList_GET_ITEM(names, i));
-            if (!value)
-                goto bad;
-            if (PyDict_SetItem(globals, name, value) < 0) {
-                Py_DECREF(value);
-                goto bad;
-            }
-        }
-    }
-    Py_DECREF(names);
-    return globals;
-    /*
-    d = PyDictProxy_New(globals);
-    Py_DECREF(globals);
-    return d;
-    */
-bad:
-    Py_XDECREF(names);
-    Py_XDECREF(globals);
-    return NULL;
-}
-''' % {'MODULE' : Naming.module_cname})
-
-pyexec_utility_code = UtilityCode(
-proto = """
-static PyObject* __Pyx_PyRun(PyObject*, PyObject*, PyObject*);
-static CYTHON_INLINE PyObject* __Pyx_PyRun2(PyObject*, PyObject*);
-""",
-impl = """
-static CYTHON_INLINE PyObject* __Pyx_PyRun2(PyObject* o, PyObject* globals) {
-    return __Pyx_PyRun(o, globals, NULL);
-}
-
-static PyObject* __Pyx_PyRun(PyObject* o, PyObject* globals, PyObject* locals) {
-    PyObject* result;
-    PyObject* s = 0;
-    char *code = 0;
-
-    if (!globals || globals == Py_None) {
-        globals = PyModule_GetDict(%s);""" % Naming.module_cname + """
-        if (!globals)
-            goto bad;
-    } else if (!PyDict_Check(globals)) {
-        PyErr_Format(PyExc_TypeError, "exec() arg 2 must be a dict, not %.100s",
-                     globals->ob_type->tp_name);
-        goto bad;
-    }
-    if (!locals || locals == Py_None) {
-        locals = globals;
-    }
-
-
-    if (PyDict_GetItemString(globals, "__builtins__") == NULL) {
-        PyDict_SetItemString(globals, "__builtins__", PyEval_GetBuiltins());
-    }
-
-    if (PyCode_Check(o)) {
-        if (PyCode_GetNumFree((PyCodeObject *)o) > 0) {
-            PyErr_SetString(PyExc_TypeError,
-                "code object passed to exec() may not contain free variables");
-            goto bad;
-        }
-        #if PY_VERSION_HEX < 0x030200B1
-        result = PyEval_EvalCode((PyCodeObject *)o, globals, locals);
-        #else
-        result = PyEval_EvalCode(o, globals, locals);
-        #endif
-    } else {
-        PyCompilerFlags cf;
-        cf.cf_flags = 0;
-        if (PyUnicode_Check(o)) {
-            cf.cf_flags = PyCF_SOURCE_IS_UTF8;
-            s = PyUnicode_AsUTF8String(o);
-            if (!s) goto bad;
-            o = s;
-        #if PY_MAJOR_VERSION >= 3
-        } else if (!PyBytes_Check(o)) {
-        #else
-        } else if (!PyString_Check(o)) {
-        #endif
-            PyErr_SetString(PyExc_TypeError,
-                "exec: arg 1 must be string, bytes or code object");
-            goto bad;
-        }
-        #if PY_MAJOR_VERSION >= 3
-        code = PyBytes_AS_STRING(o);
-        #else
-        code = PyString_AS_STRING(o);
-        #endif
-        if (PyEval_MergeCompilerFlags(&cf)) {
-            result = PyRun_StringFlags(code, Py_file_input, globals, locals, &cf);
-        } else {
-            result = PyRun_String(code, Py_file_input, globals, locals);
-        }
-        Py_XDECREF(s);
-    }
-
-    return result;
-bad:
-    Py_XDECREF(s);
-    return 0;
-}
-""")
+globals_utility_code = UtilityCode.load_cached("Globals", "Builtins.c")
+pyexec_utility_code = UtilityCode.load_cached("PyExec", "Builtins.c")
+pyexec_globals_utility_code = UtilityCode.load_cached("PyExecGlobals", "Builtins.c")
 
 intern_utility_code = UtilityCode(
 proto = """
@@ -365,16 +208,20 @@ builtin_function_table = [
                             ],
                         is_strict_signature = True)),
     BuiltinFunction('abs',        "O",    "O",     "PyNumber_Absolute"),
+    BuiltinFunction('callable',   "O",    "b",     "__Pyx_PyCallable_Check",
+                    utility_code = UtilityCode.load_cached("CallableCheck", "ObjectHandling.c")),
     #('chr',       "",     "",      ""),
     #('cmp', "",   "",     "",      ""), # int PyObject_Cmp(PyObject *o1, PyObject *o2, int *result)
     #('compile',   "",     "",      ""), # PyObject* Py_CompileString(    char *str, char *filename, int start)
     BuiltinFunction('delattr',    "OO",   "r",     "PyObject_DelAttr"),
     BuiltinFunction('dir',        "O",    "O",     "PyObject_Dir"),
     BuiltinFunction('divmod',     "OO",   "O",     "PyNumber_Divmod"),
-    BuiltinFunction('exec',       "OOO",  "O",     "__Pyx_PyRun",
+    BuiltinFunction('exec',       "OOO",  "O",     "__Pyx_PyExec3",
                     utility_code = pyexec_utility_code),
-    BuiltinFunction('exec',       "OO",   "O",     "__Pyx_PyRun2",
+    BuiltinFunction('exec',       "OO",   "O",     "__Pyx_PyExec2",
                     utility_code = pyexec_utility_code),
+    BuiltinFunction('exec',       "O",    "O",     "__Pyx_PyExecGlobals",
+                    utility_code = pyexec_globals_utility_code),
     #('eval',      "",     "",      ""),
     #('execfile',  "",     "",      ""),
     #('filter',    "",     "",      ""),
@@ -489,6 +336,8 @@ builtin_types_table = [
     ("dict",    "PyDict_Type",     [BuiltinMethod("items", "T",   "O", "PyDict_Items"),  # FIXME: Py3 mode?
                                     BuiltinMethod("keys",  "T",   "O", "PyDict_Keys"),   # FIXME: Py3 mode?
                                     BuiltinMethod("values","T",   "O", "PyDict_Values"), # FIXME: Py3 mode?
+                                    BuiltinMethod("clear", "T",   "r", "__Pyx_PyDict_Clear",
+                                                  utility_code = UtilityCode.load_cached("py_dict_clear", "Optimize.c")),
                                     BuiltinMethod("copy",  "T",   "T", "PyDict_Copy")]),
 
     ("slice",   "PySlice_Type",    [BuiltinAttribute('start'),
